@@ -3,14 +3,14 @@
 #include "vabackend.h"
 #include <stdlib.h>
 
+#if !defined(__GLIBC__)
+typedef int (*__compar_d_fn_t) (const void *, const void *, void *);
 #if defined(__FreeBSD__) && __FreeBSD__ < 14
 // https://github.com/freebsd/freebsd-src/commit/af3c78886fd8
 typedef int (*__old_compar_d_fn_t) (void *, const void *, const void *);
 #define qsort_r(base, nmemb, size, compar, thunk) \
         qsort_r(base, nmemb, size, thunk, (__old_compar_d_fn_t)compar)
-
-#elif !defined(__GLIBC__)
-typedef int (*__compar_d_fn_t) (const void *, const void *, void *);
+#endif
 #endif
 
 static const uint8_t ff_hevc_diag_scan4x4_x[16] = {
@@ -288,12 +288,12 @@ static void copyHEVCIQMatrix(NVContext *ctx, NVBuffer* buf, CUVIDPICPARAMS *picP
             ppc->ScalingList16x16[i][j] = iq->ScalingList16x16[i][pos];
 
             if (i < 2)
-                ppc->ScalingList32x32[i][j] = iq->ScalingList32x32[i * 3][pos];
+                ppc->ScalingList32x32[i][j] = iq->ScalingList32x32[i][pos];
         }
 
         ppc->ScalingListDCCoeff16x16[i] = iq->ScalingListDC16x16[i];
         if (i < 2)
-            ppc->ScalingListDCCoeff32x32[i] = iq->ScalingListDC32x32[i * 3];
+            ppc->ScalingListDCCoeff32x32[i] = iq->ScalingListDC32x32[i];
     }
 }
 
